<?php

namespace Drupal\reverse_entity_reference\Form;

use Drupal\Core\Form\ConfigFormBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Field\FieldTypePluginManagerInterface;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Field\Plugin\Field\FieldType\EntityReferenceItem;


class SettingsForm extends ConfigFormBase {

    /**
     * @var FieldTypePluginManagerInterface $fieldTypeManager
     */
    protected $fieldTypeManager;

    /**
     * {@inheritdoc}
     */
    public function __construct(ConfigFactoryInterface $config_factory, FieldTypePluginManagerInterface $fieldTypeManager) {
        parent::__construct($config_factory);
        $this->fieldTypeManager = $fieldTypeManager;
    }

    /**
    * {@inheritdoc}
    */
    public static function create(ContainerInterface $container) {
        return new static(
            $container->get('config.factory'),
            $container->get('plugin.manager.field.field_type')
        );
    }

    /**
     * {@inheritdoc}.
     */
    public function getFormId(){
        return 'reverse_entity_reference_settings_form';
    }
    /**
     * {@inheritdoc}.  
     */
    public function buildForm(array $form, FormStateInterface $form_state){
        // Form constructor
        $form = parent::buildForm($form, $form_state);
        // Default / Previous settings
        //@TODO: hide api key / client id values ... should be password field
        $config = $this->config('reverse_entity_reference.settings');
        $field_types = $this->getReferenceFieldTypes();
        $form['allowed_field_types'] = array(
            '#type' => 'select',
            '#title' => t('Allowed Field Types'),
            '#default_value' => $config->get('allowed_field_types'),
            '#options' => $field_types,
            '#multiple' => TRUE,
        );
        return $form;
    }
    
    /**
     * Gets any field types that extend from the entity_reference field.
     * i.e. entity_reference_revisions, file, image etc.
     *
     * @return string[]
     *  an array of field names field by field type id.
     */
    protected function getReferenceFieldTypes(){
        $field_definitions = $this->fieldTypeManager->getDefinitions();
        $field_definitions = array_filter( $field_definitions, function($definition) use ($field_definitions){
            $er_extension = is_subclass_of($definition['class'], $field_definitions['entity_reference']['class']);
            $is_er = $definition['class'] === $field_definitions['entity_reference']['class'];
            return( $er_extension || $is_er );
        });
        return array_combine(array_keys($field_definitions), array_map(function($definition){
            return $definition['label'];
        },$field_definitions));
    }

    /**
     * {@inheritdoc};
     */
    public function submitForm(array &$form, FormStateInterface $form_state) {
        $this->config('reverse_entity_reference.settings')
            ->set('allowed_field_types', $form_state->getValue('allowed_field_types'))
            ->save();
        return parent::submitForm($form, $form_state);
    }
    /**
     * {@inheritdoc};
     */
    public function getEditableConfigNames() {
        return array(
            'reverse_entity_reference.settings'
        );
    }

}
